Package: NCA
Type: Package
Title: Necessary Condition Analysis
Version: 5.0.2
Date: 2026-06-16
Authors@R: c(person(given = "Jan",
                    family = "Dul",
                    role = "aut",
                    email = "jdul@rsm.nl"),
             person(given = "Govert",
                    family = "Buijs",
                    role = "cre",
                    email = "buijs@rsm.nl"))
Description: Performs a Necessary Condition Analysis (NCA). (Dul, J. 2016. Necessary Condition Analysis (NCA). ''Logic and Methodology of 'Necessary but not Sufficient' causality." Organizational Research Methods 19(1), 10-52) <doi:10.1177/1094428115584005>.
  NCA identifies necessary (but not sufficient) conditions in datasets, where x causes (e.g. precedes) y. Instead of drawing a regression line ''through the middle of the data'' in an xy-plot, NCA draws the ceiling line. The ceiling line y = f(x) separates the area with observations from the area without observations.
  (Nearly) all observations are below the ceiling line: y <= f(x). The empty zone is in the upper left hand corner of the xy-plot (with the convention that the x-axis is ''horizontal'' and the y-axis is ''vertical'' and that values increase ''upwards'' and ''to the right''). The ceiling line is a (piecewise) linear non-decreasing line: a linear step function or a straight line. It indicates which level of x (e.g., an effort, a characteristic) is necessary but not sufficient for a (desired or undesired) level of y (e.g., good performance or disease). A quick start guide for using this package can be found here: <https://repub.eur.nl/pub/78323/> or <https://papers.ssrn.com/sol3/papers.cfm?abstract_id=2624981>.
URL:
        https://www.eur.nl/en/erim/erim/research-initiatives/necessary-condition-analysis
License: GPL (>= 3)
Depends: R (>= 3.5.0)
Imports: gplots, quantreg, KernSmooth, lpSolve, ggplot2, doParallel,
        foreach, iterators, plotly, truncnorm, DBI, RSQLite
NeedsCompilation: no
Repository: CRAN
Suggests: testthat
Packaged: 2026-06-17 14:11:07 UTC; govert
Author: Jan Dul [aut],
  Govert Buijs [cre]
Maintainer: Govert Buijs <buijs@rsm.nl>
Date/Publication: 2026-06-17 14:50:11 UTC
