export(nca, nca_analysis, nca_output, nca_outliers,
       nca_random, nca_power, nca_powerplot, nca_extract, nca_util_normalize,
       nca_difference,
       ceilings, line.colors, line.types, line.width, point.type, point.color)
S3method(plot, nca_result)
S3method(print, nca_result)
S3method(summary, nca_result)
S3method(print, nca_outliers)
S3method(print, nca_difference)
import(gplots)
import(ggplot2)
import(grid)
importFrom("plotly", "plot_ly", "layout", "add_trace", "add_lines")
importFrom("parallel", "detectCores")
importFrom("doParallel", "registerDoParallel", "stopImplicitCluster")
importFrom("foreach", "foreach", "%dopar%", "registerDoSEQ")
importFrom("iterators", "iter")
importFrom("lpSolve", "lp")
importFrom("KernSmooth", "dpik")
importFrom("utils", "packageDescription", "combn")
importFrom("graphics", "abline", "legend", "par", "plot", "strwidth", "title", "clip", "lines", "points", "text")
importFrom("grDevices", "dev.new", "dev.off", "pdf")
importFrom("stats", "coef", "complete.cases", "ecdf", "lm", "quantile", "aggregate")
importFrom("utils", "tail", "head", "capture.output")
importFrom("quantreg", "rq")
importFrom("stats", "rnorm", "runif", "var", "qnorm", "median", "setNames")
importFrom("grDevices", "col2rgb")
importFrom("truncnorm", "rtruncnorm")
importFrom("DBI", "dbConnect", "dbDisconnect", "dbWriteTable", "dbExecute", "dbReadTable")
importFrom("RSQLite", "SQLite")
