p_nca_ols <-
  function (loop.data, ...) {
    x <- loop.data$x
    y <- loop.data$y
    ols <- lm(y ~ x)

    intercept <- unname(coef(ols)["(Intercept)"])
    slope <- unname(coef(ols)["x"])
    line_matrix <- p_get_line_matrix(intercept, slope, loop.data)
    ceiling <- p_ceiling(loop.data, slope, intercept)
    effect <- ceiling / loop.data$scope.area
    above <- p_above(loop.data, slope, intercept)
    accuracy <- p_accuracy(loop.data, above)
    fit <- 100 * ceiling / loop.data$ce_fdh_ceiling
    ineffs <- p_ineffs(loop.data, slope, intercept)
    complexity <- 1
    metrics <- p_purity_empty

    return(list(line = ols, line_matrix = line_matrix, peers = NULL,
                slope = slope, intercept = intercept,
                ceiling = ceiling, effect = effect,
                above = above, accuracy = accuracy, fit = fit,
                ineffs = ineffs, bottleneck = NULL,
                complexity = complexity,
                noise_pct = metrics$noise_pct,
                noise_nof = metrics$noise_nof,
                exceptions_pct = metrics$exceptions_pct,
                exceptions_nof = metrics$exceptions_nof,
                support_pct = metrics$support_pct,
                support_nof = metrics$support_nof,
                spread = metrics$spread,
                sharpness = metrics$sharpness
    ))
  }
