\name{line.width}
\alias{line.width}
\docType{data}
\title{parameter defining the width of the lines in the plots}
\description{This will be used for the \bold{lwd} parameter of the plot, default is 1.5.\cr
Set before calling nca_output\cr
> line.width <- 5
}
\keyword{parameter}
